
#if PCIE9842_PRESENT == 1
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <unistd.h>
#include <pthread.h>
#include <string.h>

#include "globdef.h"
#include "uidef.h"
#include "screendef.h"
#include "rusage.h"
#include "thrdef.h"

#include "vernr.h"
#include "sdrdef.h"
#include "usbdefs.h"
#include "fft1def.h"


#define WD_AI_TRGMOD_POST      0x00   //Post Trigger Mode
#define WD_AI_TRGSRC_SOFT      0x00
#define WD_AI_TrgNegative      0x0
#define WD_IntTimeBase         0x3
#define WD_AI_ADCONVSRC_TimePacer 0
#define PCIe_9842       0x30
#define NoError 0
#define SOFTTRIG_AI      0x1
#define SYNCH_OP        1
#define ASYNCH_OP       2
#define ADL_BOOLEAN unsigned int
#define ManualSoftTrg 0x40

/* Notify when the next buffer is ready for transfer. */
//DAQ Event type for the event message  
#define DAQEnd   0
#define DBEvent  1
#define TrigEvent  2

#define MIN_PCIE9842_DIVIDEND 1
#define MAX_PCIE9842_DIVIDEND 20000



struct {
  /* One large memory chunk will be allocated for two buffers. */
  short *data_unaligned;
  short *data_aligned;
  /* The buffers are fille by the callback function in double-buffering mode. */
  short *buffer[2];
  unsigned short bufferId[2];
  short osciHandle;
  /* The number of samples contained in one buffer. */
  int m_data_size;
} digitizer_globals;

P_PCIE9842 pcie9842;
char *pcie9842_parm_text[MAX_PCIE9842_PARM]={"Rate dividend",
                                     "Resampling factor",
                                     "Freq adjust",
                                     "Check"};

char *pcie9842_parfil_name="par_pcie9842";
int halfbuf=0;
int callback_blocksize;

// These are the few functions supplied 
// by the library that we use.
short WD_AI_AsyncReTrigNextReady (u_int16_t wCardNumber, ADL_BOOLEAN *bReady, ADL_BOOLEAN *StopFlag, u_int32_t *RdyTrigCnt);
short WD_SoftTriggerGen(u_int16_t wCardNumber, u_int8_t op);
short WD_Register_Card (u_int16_t CardType, u_int16_t card_num);
short WD_AI_Config (u_int16_t wCardNumber, u_int16_t TimeBase, ADL_BOOLEAN adDutyRestore, u_int16_t ConvSrc, ADL_BOOLEAN doubleEdged, ADL_BOOLEAN AutoResetBuf);
short WD_AI_Trig_Config (u_int16_t wCardNumber, u_int16_t trigMode, u_int16_t trigSrc, u_int16_t trigPol,
                       u_int16_t anaTrigchan, double anaTriglevel, u_int32_t postTrigScans, u_int32_t preTrigScans, u_int32_t trigDelayTicks, u_int32_t reTrgCnt);
short WD_Release_Card  (u_int16_t CardNumber);
short WD_AI_ContBufferSetup (u_int16_t wCardNumber, void *pwBuffer, u_int32_t dwReadCount, u_int16_t *BufferId);
short WD_AI_EventCallBack (u_int16_t wCardNumber, short mode, short EventType, void (*callbackAddr)(int));
short WD_AI_ContReadChannel (u_int16_t CardNumber, u_int16_t Channel,
               u_int16_t BufId, u_int32_t ReadScans, u_int32_t ScanIntrv, u_int32_t SampIntrv, u_int16_t SyncMode);
short WD_AI_AsyncClear (u_int16_t CardNumber, u_int32_t *StartPos, u_int32_t *AccessCnt);               
short WD_AI_ContBufferReset (u_int16_t wCardNumber);
short WD_AI_Set_Mode (u_int16_t  wCardNumber, u_int16_t  modeCtrl, u_int16_t  wIter);
short WD_AI_AsyncDblBufferMode (u_int16_t CardNumber, ADL_BOOLEAN Enable);
short WD_AI_AsyncDblBufferHandled (u_int16_t wCardNumber);
short WD_AI_InitialMemoryAllocated (u_int16_t CardNumber, u_int32_t  *MemSize);
short WD_AI_AsyncDblBufferHalfReady (u_int16_t CardNumber, ADL_BOOLEAN *HalfReady, ADL_BOOLEAN *StopFlag);
short WD_AI_AsyncCheck (u_int16_t , ADL_BOOLEAN *Stopped, u_int32_t  *AccessCnt);
short WD_AI_CurrentViewIndex (u_int16_t wCardNumber, u_int16_t* wDblBufferIndex, u_int32_t *AccessCnt);
short WD_AI_AsyncDblBufferOverrun (u_int16_t wCardNumber, u_int16_t op, u_int16_t *overrunFlag);
 
void osciCallback (int sig_no)
{
(void)sig_no;
// This is the callback function that will be invoked by the
// driver whenever a buffer is ready for transfer.
int i, j, k, m, n;
short int *iz;
int *iw;
halfbuf=1&(halfbuf+1);
WD_AI_AsyncDblBufferHandled(digitizer_globals.osciHandle);
switch(pcie9842.resampling_factor)
  {
  case 0:
  iz=(short int*)&timf1_char[timf1p_sdr];
  for(i=0; i<digitizer_globals.m_data_size; i++)
    {
    iz[i]=digitizer_globals.buffer[halfbuf][i];
    }
  timf1p_sdr=(timf1p_sdr+sizeof(short int)*digitizer_globals.m_data_size)&timf1_bytemask;
  break;

  case 1:
  m=digitizer_globals.m_data_size/2;
  iz=(short int*)&timf1_char[timf1p_sdr];
  for(i=0; i<m; i++)
    {
    iz[i]=digitizer_globals.buffer[halfbuf][2*i  ]/2+
          digitizer_globals.buffer[halfbuf][2*i+1]/2;
    }
  timf1p_sdr=(timf1p_sdr+sizeof(short int)*m)&timf1_bytemask;
  break;

  case 2:
  m=digitizer_globals.m_data_size/4;
  iz=(short int*)&timf1_char[timf1p_sdr];
  for(i=0; i<m; i++)
    {
    iz[i]=(digitizer_globals.buffer[halfbuf][4*i  ]>>2)+
          (digitizer_globals.buffer[halfbuf][4*i+1]>>2)+
          (digitizer_globals.buffer[halfbuf][4*i+2]>>2)+
          (digitizer_globals.buffer[halfbuf][4*i+3]>>2);
    }
  timf1p_sdr=(timf1p_sdr+sizeof(short int)*m)&timf1_bytemask;
  break;

  default:
  n=16-pcie9842.resampling_factor;
  m=digitizer_globals.m_data_size/callback_blocksize;
  iw=(int*)&timf1_char[timf1p_sdr];
  for(i=0; i<m; i++)
    {
    k=digitizer_globals.buffer[halfbuf][i*callback_blocksize];
    for(j=1; j<callback_blocksize; j++)
      {
      k+=digitizer_globals.buffer[halfbuf][i*callback_blocksize+j];
      }
    iw[i]=k<<n;
    }
  timf1p_sdr=(timf1p_sdr+sizeof(int)*m)&timf1_bytemask;
  break;
  }
lir_set_event(EVENT_HWARE1_RXREADY);
}

void pcie9842_input(void)
{
#if RUSAGE_OLD == TRUE
int local_workload_counter;
#endif
int i;
int rxin_local_workload_reset;
double read_start_time, total_reads;
int timing_loop_counter,timing_loop_counter_max,initial_skip_flag;
short int err;
u_int32_t totmem;
lir_init_event(EVENT_HWARE1_RXREADY);
#if OSNUM == OSNUM_LINUX
clear_thread_times(THREAD_PCIE9842_INPUT);
#endif
#if RUSAGE_OLD == TRUE
local_workload_counter=workload_counter;
#endif
i=read_sdrpar(pcie9842_parfil_name, MAX_PCIE9842_PARM, 
                                     pcie9842_parm_text, (int*)((void*)&pcie9842));
if(i != 0 || pcie9842.check != PCIE9842PAR_VERNR)
  {
  lirerr(1359);
  goto pcie9842_init_error_exit;
  }   
if( (ui.rx_ad_speed<<pcie9842.resampling_factor) != 
                           (int)((rint)((double)200000000L/pcie9842.rate_dividend*
                                 (100000000L+(double)pcie9842.freq_adjust))/100000000L))
  {
  lirerr(1359);
  goto pcie9842_init_error_exit;
  }
callback_blocksize=1<<pcie9842.resampling_factor;
digitizer_globals.osciHandle=WD_Register_Card(PCIe_9842, 0);
if (digitizer_globals.osciHandle <0 ) 
  {
  lirerr(1361);
  goto pcie9842_init_error_exit;
  }
err=WD_AI_InitialMemoryAllocated (digitizer_globals.osciHandle, &totmem);
fprintf(stderr,"\nInitialAlloc %d bytes",totmem*1024);
if(err != NoError) 
  {
  lirerr(1367);
  goto pcie9842_error_exit;
  }
digitizer_globals.m_data_size=snd[RXAD].block_bytes/2; 
digitizer_globals.m_data_size<<=pcie9842.resampling_factor;
if(pcie9842.resampling_factor >=3)digitizer_globals.m_data_size>>=1;
make_power_of_two(&digitizer_globals.m_data_size);
while((u_int32_t)digitizer_globals.m_data_size/512 > totmem)digitizer_globals.m_data_size/=2;
digitizer_globals.data_unaligned =(short int *)malloc(2*
                      digitizer_globals.m_data_size*sizeof(short int) + 15);
if(digitizer_globals.data_unaligned == NULL) 
  {
  lirerr(1360);
  goto pcie9842_error_exit1;
  }
digitizer_globals.data_aligned=(short int *) ((MEMREF_T) ((MEMREF_T) 
       digitizer_globals.data_unaligned + (MEMREF_T) 15) & (MEMREF_T) -16);
// Save pointers to the two aligned buffers.
digitizer_globals.buffer[0] = &(digitizer_globals.data_aligned[0 * digitizer_globals.m_data_size]);
digitizer_globals.buffer[1] = &(digitizer_globals.data_aligned[1 * digitizer_globals.m_data_size]);
digitizer_globals.bufferId[0] = 0;
digitizer_globals.bufferId[1] = 0;
err=WD_AI_Config(digitizer_globals.osciHandle,
                  WD_IntTimeBase,
                  1, 
                  WD_AI_ADCONVSRC_TimePacer, 
                  0, 
                  1);
if(err != NoError)
  {
  lirerr(1365);
  goto pcie9842_error_exit;
  }  
/*
err=WD_AI_Trig_Config( digitizer_globals.osciHandle,
                       WD_AI_TRGMOD_POST,
                       WD_AI_TRGSRC_SOFT,
                       WD_AI_TrgNegative,
                       0, 0.0, 0, 0, 0, 0);
*/
if(err != NoError) 
  {
  lirerr(1362);
  goto pcie9842_error_exit;
  }  
err=WD_AI_ContBufferSetup (digitizer_globals.osciHandle,
                           digitizer_globals.buffer[0],
                           digitizer_globals.m_data_size,
                           &digitizer_globals.bufferId[0]);
if(err != NoError) 
  {
  lirerr(1363);
  goto pcie9842_error_exit;
  }  
err=WD_AI_ContBufferSetup (digitizer_globals.osciHandle,
                           digitizer_globals.buffer[1],
                           digitizer_globals.m_data_size,
                           &digitizer_globals.bufferId[1]);
if(err != NoError) 
  {
  lirerr(1363);
  goto pcie9842_error_exit;
  }  
fft1_block_timing();
fft1_hz_per_point=(float)ui.rx_ad_speed/(fft1_size*2);
if(thread_command_flag[THREAD_SCREEN]!=THRFLAG_NOT_ACTIVE)
  {
  while(thread_status_flag[THREAD_SCREEN]!=THRFLAG_ACTIVE &&
        thread_status_flag[THREAD_SCREEN]!=THRFLAG_IDLE &&
        thread_status_flag[THREAD_SCREEN]!=THRFLAG_SEM_WAIT)
    {
    if(thread_command_flag[THREAD_PCIE9842_INPUT] ==
                                       THRFLAG_KILL)goto pcie9842_error_exit;
    lir_sleep(10000);
    }
  }
err=WD_AI_EventCallBack (digitizer_globals.osciHandle, 1, DBEvent, osciCallback);
if(err != NoError) 
  {
  lirerr(1364);
  goto pcie9842_error_exit;
  }  
err=WD_AI_Set_Mode (digitizer_globals.osciHandle, ManualSoftTrg, 0);
if(err != NoError) 
  {
  lirerr(1365);
  goto pcie9842_error_exit;
  }
err=WD_AI_AsyncDblBufferMode (digitizer_globals.osciHandle, 1);
if(err != NoError) 
  {
  lirerr(1366);
  goto pcie9842_error_exit;
  }
for(i=0;i<timf1_bytes;i++)timf1_char[i]=0;
err=WD_AI_ContReadChannel(digitizer_globals.osciHandle,
                          0,
                          digitizer_globals.bufferId[0],
                          digitizer_globals.m_data_size,
                          pcie9842.rate_dividend,
                          200000000/pcie9842.rate_dividend,
                          ASYNCH_OP); //SYNCH_OP ??
if(err != NoError) 
  {
  lirerr(1364);
  goto pcie9842_error_exit;
  }
  
fprintf(stderr,"\ndigitizer_globals.m_data_size  %d",
                                              digitizer_globals.m_data_size);
#include "timing_setup.c"
thread_status_flag[THREAD_PCIE9842_INPUT]=THRFLAG_ACTIVE;
timf1p_sdr=timf1p_pa;  
while(!kill_all_flag && 
            thread_command_flag[THREAD_PCIE9842_INPUT] == THRFLAG_ACTIVE)
  {
#if RUSAGE_OLD == TRUE
  if(local_workload_counter != workload_counter)
    {
    local_workload_counter=workload_counter;
    make_thread_times(THREAD_PCIE9842_INPUT);
    }
#endif
  lir_await_event(EVENT_HWARE1_RXREADY);
  if(kill_all_flag)goto pcie9842_error_exit;
  while (!kill_all_flag && 
                  ((timf1p_sdr-timf1p_pa+timf1_bytes)&timf1_bytemask) >=
                                   snd[RXAD].block_bytes)
    {
#include "input_speed.c"
    finish_rx_read();
    }
  }
// Tell digitizer to generate no more triggers.
u_int32_t count=0, startPos=0;
WD_AI_AsyncClear(digitizer_globals.osciHandle, &startPos, &count);
lir_sleep(50000);
WD_AI_ContBufferReset(digitizer_globals.osciHandle);
WD_AI_EventCallBack (digitizer_globals.osciHandle, 0, DBEvent, NULL);
pcie9842_error_exit:;
free(digitizer_globals.data_unaligned);
pcie9842_error_exit1:;
WD_Release_Card(digitizer_globals.osciHandle);
pcie9842_init_error_exit:;
sdr=-1;
thread_status_flag[THREAD_PCIE9842_INPUT]=THRFLAG_RETURNED;
while(thread_command_flag[THREAD_PCIE9842_INPUT] != THRFLAG_NOT_ACTIVE)
  {
  lir_sleep(1000);
  }
lir_close_event(EVENT_HWARE1_RXREADY);
}

int display_pcie9842_parm_info(int *line)
{
char s[80];
int errcod;
errcod=read_sdrpar(pcie9842_parfil_name, MAX_PCIE9842_PARM, 
                               pcie9842_parm_text, 
                               (int*)((void*)&pcie9842));
if(errcod == 0)
  {
  settextcolor(7);
  sprintf(s,"Rate dividend = %i, Clk adjust = %.0f ppb  sampling rate %d Resampling %d",  
         pcie9842.rate_dividend,
         10.*pcie9842.freq_adjust,
         (int)((rint)((double)200000000L/pcie9842.rate_dividend*
                     (100000000L+(double)pcie9842.freq_adjust))/100000000L),
                     pcie9842.resampling_factor);
  lir_text(24,line[0],s);
  SNDLOG"\n%s",s);
//  line[0]+=1;
  }
return (errcod);
}

void init_pcie9842(void)
{
FILE *pcie9842_file;

int i, line;
char s[80];
int *sdr_pi;
short handle;
// One or two cards.
handle=WD_Register_Card(PCIe_9842, 0);
if(handle < 0 ) 
  {
  lir_text(5,5,"PCIe 9842 not found.");
  lir_text(5,7,press_any_key);
  await_keyboard();
  return;
  }
line=2;  
// Look for more cards and allow the user to choose
// one or two cards.
sprintf(s,"Set rate dividend (%d to %d) =>",
                                    MIN_PCIE9842_DIVIDEND,
                                    MAX_PCIE9842_DIVIDEND);
lir_text(3,line,s);
pcie9842.rate_dividend=lir_get_integer(4+strlen(s),line,7,
                                     MIN_PCIE9842_DIVIDEND,
                                     MAX_PCIE9842_DIVIDEND);
line+=2;
if(kill_all_flag)
  {
  WD_Release_Card(handle);
  return;
  }
lir_text(3,line,"Enter sampling clock error in ppb =>");
pcie9842.freq_adjust=0.1*lir_get_float(40,line,9,-300000.,300000.);
line+=2;
lir_text(3,line,"Enter resampling factor in powers of two (0-4) =>");
pcie9842.resampling_factor=lir_get_integer(53,line,1,0,4);
// set up other parameters and verify that the card(s) 
// are OK. Store all we neet to know in the pcie9842 structure.
pcie9842_file=fopen(pcie9842_parfil_name,"w");
if(pcie9842_file == NULL)
  {
  lirerr(382268);
  return;
  }
pcie9842.check=PCIE9842PAR_VERNR;
WD_Release_Card(handle);
sdr_pi=(int*)(&pcie9842);
for(i=0; i<MAX_PCIE9842_PARM; i++)
  {
  fprintf(pcie9842_file,"%s [%d]\n",pcie9842_parm_text[i],sdr_pi[i]);
  }
parfile_end(pcie9842_file);
ui.rx_addev_no=PCIE9842_DEVICE_CODE;
ui.rx_ad_speed=(int)((rint)((double)200000000L/pcie9842.rate_dividend*
                     (100000000L+(double)pcie9842.freq_adjust))/100000000L);
ui.rx_ad_speed>>=pcie9842.resampling_factor;
if(pcie9842.resampling_factor <=2)
  {
  ui.rx_input_mode=0;
  }
else
  {
  ui.rx_input_mode=DWORD_INPUT;
  }
ui.rx_rf_channels=1;
ui.rx_ad_channels=1;
ui.rx_admode=0;
}

#endif
